package com.redhat.jvahala.selectionorderreproducer;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.junit.Test;
import org.optaplanner.core.api.solver.SolverFactory;
import org.optaplanner.core.impl.localsearch.scope.LocalSearchStepScope;
import org.optaplanner.core.impl.phase.event.PhaseLifecycleListener;
import org.optaplanner.core.impl.phase.scope.AbstractPhaseScope;
import org.optaplanner.core.impl.phase.scope.AbstractStepScope;
import org.optaplanner.core.impl.solver.DefaultSolver;
import org.optaplanner.core.impl.solver.scope.DefaultSolverScope;
import org.optaplanner.examples.tsp.domain.TravelingSalesmanTour;
import org.optaplanner.examples.tsp.domain.location.Location;
import org.optaplanner.examples.tsp.persistence.TspDao;

public class AppTest {

    public class StepsCountListener implements PhaseLifecycleListener {

        private final Map<String, Integer> stepMap = new HashMap<String, Integer>();

        @Override
        public void stepStarted(AbstractStepScope stepScope) {
        }

        @Override
        public void stepEnded(AbstractStepScope stepScope) {
            String string = ((LocalSearchStepScope) stepScope).getStepString();
            if (stepMap.containsKey(string)) {
                stepMap.put(string, stepMap.get(string) + 1);
            } else {
                stepMap.put(string, 1);
            }
        }

        public Map<String, Integer> getStepMap() {
            return stepMap;
        }

        public void phaseStarted(AbstractPhaseScope phaseScope) {
        }

        public void phaseEnded(AbstractPhaseScope phaseScope) {
        }

        public void solvingStarted(DefaultSolverScope solverScope) {
        }

        public void solvingEnded(DefaultSolverScope solverScope) {
        }
    }

    @Test
    public void reproduce() {
        TravelingSalesmanTour problem
                = (TravelingSalesmanTour) (new TspDao().readSolution(new File("data/tsp/line_10.xml")));
        List<Location> locations = problem.getLocationList();
        DefaultSolver solver = (DefaultSolver) SolverFactory.createFromXmlResource("constructionHeuristics.xml").buildSolver();
        solver.solve(problem);

        TravelingSalesmanTour problemAfterCH = (TravelingSalesmanTour) solver.getBestSolution();

        solver = (DefaultSolver) SolverFactory.createFromXmlResource("nearbySelection.xml").buildSolver();

        StepsCountListener listener = new StepsCountListener();
        int numberOfInstances = 10000;
        for (int i = 0; i < numberOfInstances; i++) {
            solver.addPhaseLifecycleListener(listener);
            solver.solve(problemAfterCH);
        }
        for (Map.Entry<String, Integer> entry : listener.getStepMap().entrySet()) {
            System.out.println("Move: " + entry.getKey() + " and it's count: " + entry.getValue());
        }
    }
}
